from typing import Type

from django.apps import apps
from django.core.checks import Error, Tags, register

from . import settings
from .dashboard import AppIndexDashboard, Dashboard
from .utils import get_app_dashboard_config, import_value


def check_dashboard(key, path, base_cls):
    try:
        dashboard_csl: Type[base_cls] = import_value(path)
    except ImportError:
        error = Error(
            f"Failed to import {key}: {path!r}",
            hint=f"Fix {key} value in settings.",
            id='jet.dashboard.E001',
        )
        return [error]
    if not issubclass(dashboard_csl, base_cls):
        error = Error(
            f"Invalid {key}. "
            f"Expected subclass of {base_cls.__name__} got {type(dashboard_csl).__name__}",
            id='jet.dashboard.E002',
        )
        return [error]
    return []


@register(Tags.compatibility)
def check_index_dashboard(app_configs, **kwargs):
    return check_dashboard('JET_INDEX_DASHBOARD', settings.JET_INDEX_DASHBOARD, Dashboard)


@register()
def check_app_index_dashboard(app_configs, **kwargs):
    app_labels = {a.label for a in apps.get_app_configs()}
    errors = []
    config = get_app_dashboard_config(settings.JET_APP_INDEX_DASHBOARD)
    for key in config.keys():
        if key is None:
            continue
        if key not in app_labels:
            errors.append(
                Error(
                    f"{key} is not in the app list.",
                    hint=f"Choose from {', '.join(app_labels)}.",
                    id='jet.dashboard.E003',
                )
            )
    for key, path in config.items():
        es = check_dashboard(f'JET_APP_INDEX_DASHBOARD[{key}]', path, AppIndexDashboard)
        errors.extend(es)
    return errors


@register(Tags.compatibility)
def check_yandex_metrika_creds(app_configs, **kwargs):
    if (
        settings.JET_MODULE_YANDEX_METRIKA_CLIENT_ID and
        not settings.JET_MODULE_YANDEX_METRIKA_CLIENT_SECRET or
        settings.JET_MODULE_YANDEX_METRIKA_CLIENT_SECRET and
        not settings.JET_MODULE_YANDEX_METRIKA_CLIENT_ID
    ):
        error = Error(
            f"You need to specify both client_id and client_secret for yandex metrika.",
            id='jet.dashboard.E003',
        )
        return [error]
    return []



# -*- coding: utf-8 -*-
from django.db import models, migrations
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Bookmark',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('url', models.URLField(verbose_name='URL')),
                ('title', models.CharField(max_length=255, verbose_name='title')),
                ('user', models.PositiveIntegerField(verbose_name='user')),
                ('date_add', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date created')),
            ],
            options={
                'ordering': ('date_add',),
                'verbose_name': 'bookmark',
                'verbose_name_plural': 'bookmarks',
            },
        ),
        migrations.CreateModel(
            name='PinnedApplication',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('app_label', models.CharField(max_length=255, verbose_name='application name')),
                ('user', models.PositiveIntegerField(verbose_name='user')),
                ('date_add', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date created')),
            ],
            options={
                'ordering': ('date_add',),
                'verbose_name': 'pinned application',
                'verbose_name_plural': 'pinned applications',
            },
        ),
        migrations.CreateModel(
            name='UserDashboardModule',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('module', models.CharField(max_length=255, verbose_name='module')),
                ('app_label', models.CharField(max_length=255, null=True, verbose_name='application name', blank=True)),
                ('user', models.PositiveIntegerField(verbose_name='user')),
                ('column', models.PositiveIntegerField(verbose_name='column')),
                ('order', models.IntegerField(verbose_name='order')),
                ('settings', models.TextField(default=b'', verbose_name='settings', blank=True)),
                ('children', models.TextField(default=b'', verbose_name='children', blank=True)),
                ('collapsed', models.BooleanField(default=False, verbose_name='collapsed')),
            ],
            options={
                'ordering': ('column', 'order'),
                'verbose_name': 'user dashboard module',
                'verbose_name_plural': 'user dashboard modules',
            },
        ),
    ]



import datetime

from django import forms
from django.utils.translation import gettext_lazy as _

from .base import DashboardModule


class FeedSettingsForm(forms.Form):
    limit = forms.IntegerField(label=_('Items limit'), min_value=1)
    feed_url = forms.URLField(label=_('Feed URL'))


class Feed(DashboardModule):
    """
    Display RSS Feed entries with following information:
    entry title, date and link to the full version

    Usage example:

    .. code-block:: python

        from django.utils.translation import gettext_lazy as _
        from jet.dashboard import modules
        from jet.dashboard.dashboard import Dashboard, AppIndexDashboard


        class CustomIndexDashboard(Dashboard):
            columns = 3

            def init_with_context(self, context):
                self.children.append(modules.Feed(
                    _('Latest Django News'),
                    feed_url='http://www.djangoproject.com/rss/weblog/',
                    limit=5,
                    column=0,
                    order=0
                ))

    """

    title = _('RSS Feed')
    template = 'jet.dashboard/modules/feed.html'

    #: URL of the RSS feed (may be changed by each user personally).
    feed_url = None

    #: Number if entries to be shown (may be changed by each user personally).
    limit = None
    settings_form = FeedSettingsForm
    ajax_load = True

    def __init__(self, title=None, feed_url=None, limit=None, **kwargs):
        kwargs.update({'feed_url': feed_url, 'limit': limit})
        super(Feed, self).__init__(title, **kwargs)

    def settings_dict(self):
        return {
            'feed_url': self.feed_url,
            'limit': self.limit
        }

    def load_settings(self, settings):
        self.feed_url = settings.get('feed_url')
        self.limit = settings.get('limit')

    def init_with_context(self, context):
        if self.feed_url is not None:
            try:
                import feedparser

                feed = feedparser.parse(self.feed_url)

                if self.limit is not None:
                    entries = feed['entries'][:self.limit]
                else:
                    entries = feed['entries']

                for entry in entries:
                    try:
                        entry.date = datetime.date(*entry.published_parsed[0:3])
                    except:
                        pass

                    self.children.append(entry)
            except ImportError:
                self.children.append({
                    'title': _('You must install the FeedParser python module'),
                    'warning': True,
                })
        else:
            self.children.append({
                'title': _('You must provide a valid feed URL'),
                'warning': True,
            })



from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _


class Bookmark(models.Model):
    url = models.URLField(verbose_name=_('URL'))
    title = models.CharField(verbose_name=_('title'), max_length=255)
    user = models.PositiveIntegerField(verbose_name=_('user'))
    date_add = models.DateTimeField(verbose_name=_('date created'), default=timezone.now)

    class Meta:
        verbose_name = _('bookmark')
        verbose_name_plural = _('bookmarks')
        ordering = ('date_add',)

    def __str__(self):
        return self.title


class PinnedApplication(models.Model):
    app_label = models.CharField(verbose_name=_('application name'), max_length=255)
    user = models.PositiveIntegerField(verbose_name=_('user'))
    date_add = models.DateTimeField(verbose_name=_('date created'), default=timezone.now)

    class Meta:
        verbose_name = _('pinned application')
        verbose_name_plural = _('pinned applications')
        ordering = ('date_add',)

    def __str__(self):
        return self.app_label



import logging
from typing import Union

from jet.dashboard import settings
from jet.utils import import_value


logger = logging.getLogger(__name__)


def get_app_dashboard_config(config: Union[str, dict]):
    if isinstance(config, str):
        return {None: config}
    return config


def get_app_dashboard(app: str, config: dict):
    if app in config:
        return config[app]
    if None in config:
        return config[None]
    return settings.JET_DEFAULT_APP_INDEX_DASHBOARD


def get_current_dashboard(app_label=None):
    if app_label is None:
        path = settings.JET_INDEX_DASHBOARD
    else:
        config = get_app_dashboard_config(settings.JET_APP_INDEX_DASHBOARD)
        path = get_app_dashboard(app_label, config)
    dashboard_cls = import_value(path)
    return dashboard_cls



from django.utils.translation import gettext_lazy as _

from jet.utils import get_app_list
from .base import DashboardModule


class AppList(DashboardModule):
    """
    Shows applications and containing models links. For each model "created" and "change" links are displayed.

    Usage example:

    .. code-block:: python

        from django.utils.translation import gettext_lazy as _
        from jet.dashboard import modules
        from jet.dashboard.dashboard import Dashboard, AppIndexDashboard


        class CustomIndexDashboard(Dashboard):
            columns = 3

            def init_with_context(self, context):
                self.children.append(modules.AppList(
                    _('Applications'),
                    exclude=('auth.*',),
                    column=0,
                    order=0
                ))

    """

    title = _('Applications')
    template = 'jet.dashboard/modules/app_list.html'

    #: Specify models which should be displayed. ``models`` is an array of string formatted as ``app_label.model``.
    #: Also its possible to specify all application models with * sign (e.g. ``auth.*``).
    models = None

    #: Specify models which should NOT be displayed. ``exclude`` is an array of string formatted as ``app_label.model``.
    #: Also its possible to specify all application models with * sign (e.g. ``auth.*``).
    exclude = None
    hide_empty = True

    def settings_dict(self):
        return {
            'models': self.models,
            'exclude': self.exclude
        }

    def load_settings(self, settings):
        self.models = settings.get('models')
        self.exclude = settings.get('exclude')

    def init_with_context(self, context):
        app_list = get_app_list(context)
        app_to_remove = []

        for app in app_list:
            app_name = app.get('app_label', app.get('name', ''))
            app['models'] = filter(
                lambda model: self.models is None or ('%s.%s' % (app_name, model['object_name'])) in self.models or (
                        '%s.*' % app_name) in self.models,
                app['models']
            )
            app['models'] = filter(
                lambda model: self.exclude is None or (
                        ('%s.%s' % (app_name, model['object_name'])) not in self.exclude and (
                            '%s.*' % app_name) not in self.exclude),
                app['models']
            )
            app['models'] = list(app['models'])

            if self.hide_empty and len(list(app['models'])) == 0:
                app_to_remove.append(app)

        for app in app_to_remove:
            app_list.remove(app)

        self.children = app_list



import json
import logging

from dal_select2.views import Select2QuerySetView
from django.utils.http import urlencode
from django.views.decorators.http import require_POST

from jet.forms import AddBookmarkForm, ModelLookupForm, RemoveBookmarkForm, ToggleApplicationPinForm
from jet.models import Bookmark
from jet.utils import JsonResponse


logger = logging.getLogger(__name__)


@require_POST
def add_bookmark_view(request):
    result = {'error': False}
    form = AddBookmarkForm(request, request.POST)

    if form.is_valid():
        bookmark = form.save()
        result.update({
            'id': bookmark.pk,
            'title': bookmark.title,
            'url': bookmark.url
        })
    else:
        result['error'] = True

    return JsonResponse(result)


@require_POST
def remove_bookmark_view(request):
    result = {'error': False}

    try:
        instance = Bookmark.objects.get(pk=request.POST.get('id'))
        form = RemoveBookmarkForm(request, request.POST, instance=instance)

        if form.is_valid():
            form.save()
        else:
            result['error'] = True
    except Bookmark.DoesNotExist:
        result['error'] = True

    return JsonResponse(result)


@require_POST
def toggle_application_pin_view(request):
    result = {'error': False}
    form = ToggleApplicationPinForm(request, request.POST)

    if form.is_valid():
        pinned = form.save()
        result['pinned'] = pinned
    else:
        result['error'] = True

    return JsonResponse(result)


class ModelLookupView(Select2QuerySetView):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.form = None

    def get_queryset(self):
        self.form = ModelLookupForm(self.request, self.request.GET)
        if self.form.is_valid():
            return self.form.get_queryset()
        else:
            logger.debug(self.form.errors.as_data())
            return self.form.model_cls.objects.none()

    def get_results(self, context):
        if self.form and hasattr(self.form, 'cleaned_data'):
            form = self.form
        else:
            form = ModelLookupForm(self.request, self.request.GET)
            # just populate cleaned_data,
            # error should be caught in get_queryset
            form.is_valid()
        out = [
            {
                'id': self.get_result_value(obj),
                'text': self.get_result_label(obj),
            }
            for obj in context['object_list']
        ]
        lookup_kwarg = form.cleaned_data['lookup_kwarg']
        params = form.cleaned_data['lookup_params']
        if lookup_kwarg and params:
            params = json.loads(params)
            for obj in out:
                p = params.copy()
                p[lookup_kwarg] = obj['id']
                obj['url'] = '?%s' % urlencode(sorted(p.items()))
        return out


model_lookup_view = ModelLookupView.as_view()



from django.conf import settings


def _get(key, default=None):
    return getattr(settings, key, default)


# dashboard
JET_DEFAULT_INDEX_DASHBOARD = 'jet.dashboard.dashboard.DefaultIndexDashboard'
JET_INDEX_DASHBOARD = _get('JET_INDEX_DASHBOARD', JET_DEFAULT_INDEX_DASHBOARD)
# app dashboard
JET_DEFAULT_APP_INDEX_DASHBOARD = 'jet.dashboard.dashboard.DefaultAppIndexDashboard'
JET_APP_INDEX_DASHBOARD = _get('JET_APP_INDEX_DASHBOARD', JET_DEFAULT_APP_INDEX_DASHBOARD)

# google analytics
JET_MODULE_GOOGLE_ANALYTICS_CLIENT_SECRETS_FILE = _get('JET_MODULE_GOOGLE_ANALYTICS_CLIENT_SECRETS_FILE')

# yandex metrika
JET_MODULE_YANDEX_METRIKA_CLIENT_ID = _get('JET_MODULE_YANDEX_METRIKA_CLIENT_ID')
JET_MODULE_YANDEX_METRIKA_CLIENT_SECRET = _get('JET_MODULE_YANDEX_METRIKA_CLIENT_SECRET')



from django.urls import re_path

from . import views


urlpatterns = [
    re_path(
        r'^google-analytics/grant/(?P<pk>\d+)/$',
        views.google_analytics_grant_view,
        name='google-analytics-grant',
    ),
    re_path(
        r'^google-analytics/revoke/(?P<pk>\d+)/$',
        views.google_analytics_revoke_view,
        name='google-analytics-revoke',
    ),
    re_path(
        r'^google-analytics/callback/',
        views.google_analytics_callback_view,
        name='google-analytics-callback',
    ),
]



# -*- coding: utf-8 -*-
from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='UserDashboardModule',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
                ('title', models.CharField(verbose_name='Title', max_length=255)),
                ('module', models.CharField(verbose_name='module', max_length=255)),
                ('app_label', models.CharField(verbose_name='application name', max_length=255, blank=True, null=True)),
                ('user', models.PositiveIntegerField(verbose_name='user')),
                ('column', models.PositiveIntegerField(verbose_name='column')),
                ('order', models.IntegerField(verbose_name='order')),
                ('settings', models.TextField(verbose_name='settings', blank=True, default='')),
                ('children', models.TextField(verbose_name='children', blank=True, default='')),
                ('collapsed', models.BooleanField(verbose_name='collapsed', default=False)),
            ],
            options={
                'verbose_name': 'user dashboard module',
                'verbose_name_plural': 'user dashboard modules',
                'ordering': ('column', 'order'),
            },
        ),
    ]



import datetime
import json
import logging
from importlib import import_module

from django.apps.registry import apps
from django.contrib import admin, messages
from django.contrib.admin import AdminSite
from django.contrib.admin.options import IncorrectLookupParameters
from django.core.serializers.json import DjangoJSONEncoder
from django.http import HttpResponse
from django.template import Context
from django.urls import NoReverseMatch, resolve, reverse
from django.utils import translation
from django.utils.encoding import force_text, smart_str
from django.utils.functional import Promise
from django.utils.text import capfirst


logger = logging.getLogger(__name__)


class JsonResponse(HttpResponse):
    """
    An HTTP response class that consumes data to be serialized to JSON.
    :param data: Data to be dumped into json. By default only ``dict`` objects
      are allowed to be passed due to a security flaw before EcmaScript 5. See
      the ``safe`` parameter for more information.
    :param encoder: Should be an json encoder class. Defaults to
      ``django.core.serializers.json.DjangoJSONEncoder``.
    :param safe: Controls if only ``dict`` objects may be serialized. Defaults
      to ``True``.
    """

    def __init__(self, data, encoder=DjangoJSONEncoder, safe=True, **kwargs):
        if safe and not isinstance(data, dict):
            raise TypeError('In order to allow non-dict objects to be '
                            'serialized set the safe parameter to False')
        kwargs.setdefault('content_type', 'application/json')
        data = json.dumps(data, cls=encoder)
        super(JsonResponse, self).__init__(content=data, **kwargs)


def get_app_list(context, order=True):
    admin_site = get_admin_site(context)
    request = context['request']

    app_dict = {}
    for model, model_admin in admin_site._registry.items():
        app_label = model._meta.app_label
        has_module_perms = model_admin.has_module_permission(request)

        if has_module_perms:
            perms = model_admin.get_model_perms(request)

            # Check whether user has any perm for this module.
            # If so, add the module to the model_list.
            if True in perms.values():
                info = (app_label, model._meta.model_name)
                model_dict = {
                    'name': capfirst(model._meta.verbose_name_plural),
                    'object_name': model._meta.object_name,
                    'perms': perms,
                    'model_name': model._meta.model_name
                }
                if perms.get('view', False) or perms.get('change', False):
                    try:
                        model_dict['admin_url'] = reverse('admin:%s_%s_changelist' % info, current_app=admin_site.name)
                    except NoReverseMatch:
                        pass
                if perms.get('add', False):
                    try:
                        model_dict['add_url'] = reverse('admin:%s_%s_add' % info, current_app=admin_site.name)
                    except NoReverseMatch:
                        pass
                if app_label in app_dict:
                    app_dict[app_label]['models'].append(model_dict)
                else:
                    try:
                        name = apps.get_app_config(app_label).verbose_name
                    except NameError:
                        name = app_label.title()
                    app_dict[app_label] = {
                        'name': name,
                        'app_label': app_label,
                        'app_url': reverse(
                            'admin:app_list',
                            kwargs={'app_label': app_label},
                            current_app=admin_site.name,
                        ),
                        'has_module_perms': has_module_perms,
                        'models': [model_dict],
                    }

    # Sort the apps alphabetically.
    app_list = list(app_dict.values())

    if order:
        app_list.sort(key=lambda x: x['name'].lower())

        # Sort the models alphabetically within each app.
        for app in app_list:
            app['models'].sort(key=lambda x: x['name'])

    return app_list


def get_admin_site(context):
    try:
        current_resolver = resolve(context.get('request').path)
        index_resolver = resolve(reverse('%s:index' % current_resolver.namespaces[0]))

        if hasattr(index_resolver.func, 'admin_site'):
            return index_resolver.func.admin_site

        for func_closure in index_resolver.func.__closure__:
            if isinstance(func_closure.cell_contents, AdminSite):
                return func_closure.cell_contents
    except Exception as e:
        logger.debug(e)

    return admin.site


def get_admin_site_name(context):
    return get_admin_site(context).name


class LazyDateTimeEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime.datetime) or isinstance(obj, datetime.date):
            return obj.isoformat()
        elif isinstance(obj, Promise):
            return force_text(obj)
        return self.encode(obj)


def get_model_instance_label(instance):
    if getattr(instance, "related_label", None):
        return instance.related_label()
    return smart_str(instance)


class SuccessMessageMixin:
    """
    Adds a success message on successful form submission.
    """
    success_message = ''

    def form_valid(self, form):
        response = super().form_valid(form)
        success_message = self.get_success_message(form.cleaned_data)
        if success_message:
            messages.success(self.request, success_message)
        return response

    def get_success_message(self, cleaned_data):
        return self.success_message % cleaned_data


def get_model_queryset(admin_site, model, request, preserved_filters=None):
    model_admin = admin_site._registry.get(model)

    if model_admin is None:
        return

    try:
        changelist_url = reverse('%s:%s_%s_changelist' % (
            admin_site.name,
            model._meta.app_label,
            model._meta.model_name
        ))
    except NoReverseMatch:
        return

    changelist_filters = None

    if preserved_filters:
        changelist_filters = preserved_filters.get('_changelist_filters')

    if changelist_filters:
        changelist_url += '?' + changelist_filters

    if model_admin:
        queryset = model_admin.get_queryset(request)
    else:
        queryset = model.objects

    list_display = model_admin.get_list_display(request)
    list_display_links = model_admin.get_list_display_links(request, list_display)
    list_filter = model_admin.get_list_filter(request)
    search_fields = model_admin.get_search_fields(request) \
        if hasattr(model_admin, 'get_search_fields') else model_admin.search_fields
    list_select_related = model_admin.get_list_select_related(request) \
        if hasattr(model_admin, 'get_list_select_related') else model_admin.list_select_related

    actions = model_admin.get_actions(request)
    if actions:
        list_display = ['action_checkbox'] + list(list_display)

    ChangeList = model_admin.get_changelist(request)

    change_list_args = [
        request, model, list_display, list_display_links, list_filter,
        model_admin.date_hierarchy, search_fields, list_select_related,
        model_admin.list_per_page, model_admin.list_max_show_all,
        model_admin.list_editable, model_admin]

    try:
        sortable_by = model_admin.get_sortable_by(request)
        change_list_args.append(sortable_by)
    except AttributeError:
        # django version < 2.1
        pass

    try:
        cl = ChangeList(*change_list_args)
        queryset = cl.get_queryset(request)
    except IncorrectLookupParameters:
        pass

    return queryset


def get_possible_language_codes():
    language_code = translation.get_language()

    language_code = language_code.replace('_', '-').lower()
    language_codes = []

    # making dialect part uppercase
    split = language_code.split('-', 2)
    if len(split) == 2:
        language_code = '%s-%s' % (split[0].lower(), split[1].upper()) if split[0] != split[1] else split[0]

    language_codes.append(language_code)

    # adding language code without dialect part
    if len(split) == 2:
        language_codes.append(split[0].lower())

    return language_codes


def context_to_dict(context):
    if isinstance(context, Context):
        flat = {}
        for d in context.dicts:
            flat.update(d)
        context = flat

    return context


def user_is_authenticated(user):
    if callable(user.is_authenticated):
        return user.is_authenticated()
    else:
        return user.is_authenticated


def format_widget_data(data: dict):
    res = {}
    for key, value in data.items():
        if value is None:
            continue
        if isinstance(value, bool):
            value = str(value).lower()
        key = key.replace('_', '-')
        key = f'data-{key}'
        res[key] = value
    return res


def import_value(path: str):
    module, name = path.rsplit('.', 1)
    try:
        module = import_module(module)
        value = getattr(module, name)
    except (ImportError, AttributeError) as e:
        raise ImportError(e)
    return value



import json
from importlib import import_module

from django.db import models
from django.utils.translation import gettext_lazy as _

from jet.utils import LazyDateTimeEncoder


class UserDashboardModule(models.Model):
    title = models.CharField(verbose_name=_('Title'), max_length=255)
    module = models.CharField(verbose_name=_('module'), max_length=255)
    app_label = models.CharField(verbose_name=_('application name'), max_length=255, null=True, blank=True)
    user = models.PositiveIntegerField(verbose_name=_('user'))
    column = models.PositiveIntegerField(verbose_name=_('column'))
    order = models.IntegerField(verbose_name=_('order'))
    settings = models.TextField(verbose_name=_('settings'), default='', blank=True)
    children = models.TextField(verbose_name=_('children'), default='', blank=True)
    collapsed = models.BooleanField(verbose_name=_('collapsed'), default=False)

    class Meta:
        verbose_name = _('user dashboard module')
        verbose_name_plural = _('user dashboard modules')
        ordering = ('column', 'order')

    def __str__(self):
        return self.module

    def load_module(self):
        try:
            package, module_name = self.module.rsplit('.', 1)
            package = import_module(package)
            module = getattr(package, module_name)

            return module
        except AttributeError:
            return None
        except ImportError:
            return None

    def pop_settings(self, pop_settings):
        settings = json.loads(self.settings)

        for setting in pop_settings:
            if setting in settings:
                settings.pop(setting)

        self.settings = json.dumps(settings, cls=LazyDateTimeEncoder)
        self.save()

    def update_settings(self, update_settings):
        settings = json.loads(self.settings)

        settings.update(update_settings)

        self.settings = json.dumps(settings, cls=LazyDateTimeEncoder)
        self.save()



import datetime
import json
from urllib import request
from urllib.error import HTTPError, URLError
from urllib.parse import urlencode

from django import forms
from django.forms import Widget
from django.urls import reverse
from django.utils import formats
from django.utils.encoding import force_text
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.text import capfirst
from django.utils.translation import gettext_lazy as _

from jet.dashboard.settings import JET_MODULE_YANDEX_METRIKA_CLIENT_ID, JET_MODULE_YANDEX_METRIKA_CLIENT_SECRET
from ..base import DashboardModule


class YandexMetrikaClient:
    OAUTH_BASE_URL = 'https://oauth.yandex.ru/'
    API_BASE_URL = 'https://api-metrika.yandex.ru/'
    CLIENT_ID = JET_MODULE_YANDEX_METRIKA_CLIENT_ID
    CLIENT_SECRET = JET_MODULE_YANDEX_METRIKA_CLIENT_SECRET

    def __init__(self, access_token=None):
        self.access_token = access_token

    def request(self, base_url, url, data=None, headers=None):
        url = '%s%s' % (base_url, url)

        if data is not None:
            data = urlencode(data).encode()

        if headers is None:
            headers = {}

        req = request.Request(url, data, headers)

        try:
            f = request.urlopen(req)
            result = f.read().decode('utf8')
            result = json.loads(result)
        except URLError as e:
            return None, e

        return result, None

    def get_oauth_authorize_url(self, state=''):
        return '%sauthorize' \
               '?response_type=code' \
               '&state=%s' \
               '&client_id=%s' % (self.OAUTH_BASE_URL, state, self.CLIENT_ID)

    def oauth_request(self, url, data=None):
        return self.request(self.OAUTH_BASE_URL, url, data)

    def oath_token_request(self, code):
        data = {
            'grant_type': 'authorization_code',
            'code': code,
            'client_id': self.CLIENT_ID,
            'client_secret': self.CLIENT_SECRET
        }
        return self.oauth_request('token', data)

    def api_request(self, url, data=None):
        headers = None
        if self.access_token is not None:
            headers = {'Authorization': 'OAuth %s' % self.access_token}
        return self.request(self.API_BASE_URL, url, data, headers)

    def api_counters_request(self):
        return self.api_request('counters.json')

    def api_stat_traffic_summary(self, counter, date1, date2, group=None):
        if group is None:
            group = 'day'
        return self.api_request('stat/traffic/summary.json?id=%s&date1=%s&date2=%s&group=%s' % (
            counter,
            date1.strftime('%Y%m%d'),
            date2.strftime('%Y%m%d'),
            group
        ))


class AccessTokenWidget(Widget):
    module = None

    def render(self, name, value, attrs=None, renderer=None):
        if value and len(value) > 0:
            link = '<a href="%s">%s</a>' % (
                reverse('jet-dashboard:yandex-metrika-revoke', kwargs={'pk': self.module.model.pk}),
                force_text(_('Revoke access'))
            )
        else:
            link = '<a href="%s">%s</a>' % (
                reverse('jet-dashboard:yandex-metrika-grant', kwargs={'pk': self.module.model.pk}),
                force_text(_('Grant access'))
            )

        if value is None:
            value = ''

        return format_html('%s<input type="hidden" name="access_token" value="%s">' % (link, value))


class YandexMetrikaSettingsForm(forms.Form):
    access_token = forms.CharField(label=_('Access'), widget=AccessTokenWidget)
    counter = forms.ChoiceField(label=_('Counter'))
    period = forms.ChoiceField(label=_('Statistics period'), choices=(
        (0, _('Today')),
        (6, _('Last week')),
        (30, _('Last month')),
        (31 * 3 - 1, _('Last quarter')),
        (364, _('Last year')),
    ))

    def set_module(self, module):
        self.fields['access_token'].widget.module = module
        self.set_counter_choices(module)

    def set_counter_choices(self, module):
        counters = module.counters()
        if counters is not None:
            self.fields['counter'].choices = (('', '-- %s --' % force_text(_('none'))),)
            self.fields['counter'].choices.extend(map(lambda x: (x['id'], x['site']), counters))
        else:
            label = force_text(_('grant access first')) if module.access_token is None else force_text(
                _('counters loading failed'))
            self.fields['counter'].choices = (('', '-- %s -- ' % label),)


class YandexMetrikaChartSettingsForm(YandexMetrikaSettingsForm):
    show = forms.ChoiceField(label=_('Show'), choices=(
        ('visitors', capfirst(_('visitors'))),
        ('visits', capfirst(_('visits'))),
        ('page_views', capfirst(_('views'))),
    ))
    group = forms.ChoiceField(label=_('Group'), choices=(
        ('day', _('By day')),
        ('week', _('By week')),
        ('month', _('By month')),
    ))


class YandexMetrikaPeriodVisitorsSettingsForm(YandexMetrikaSettingsForm):
    group = forms.ChoiceField(label=_('Group'), choices=(
        ('day', _('By day')),
        ('week', _('By week')),
        ('month', _('By month')),
    ))


class YandexMetrikaBase(DashboardModule):
    settings_form = YandexMetrikaSettingsForm
    ajax_load = True
    contrast = True
    period = None
    access_token = None
    expires_in = None
    token_type = None
    counter = None
    error = None

    def settings_dict(self):
        return {
            'period': self.period,
            'access_token': self.access_token,
            'expires_in': self.expires_in,
            'token_type': self.token_type,
            'counter': self.counter
        }

    def load_settings(self, settings):
        try:
            self.period = int(settings.get('period'))
        except TypeError:
            self.period = 0
        self.access_token = settings.get('access_token')
        self.expires_in = settings.get('expires_in')
        self.token_type = settings.get('token_type')
        self.counter = settings.get('counter')

    def init_with_context(self, context):
        raise NotImplementedError('subclasses of YandexMetrika must provide a init_with_context() method')

    def counters(self):
        client = YandexMetrikaClient(self.access_token)
        counters, exception = client.api_counters_request()

        if counters is not None:
            return counters['counters']
        else:
            return None

    def format_grouped_date(self, date, group):
        if group == 'week':
            date = u'%s — %s' % (
                (date - datetime.timedelta(days=7)).strftime('%d.%m'),
                date.strftime('%d.%m')
            )
        elif group == 'month':
            date = date.strftime('%b, %Y')
        else:
            date = formats.date_format(date, 'DATE_FORMAT')
        return date

    def counter_attached(self):
        if self.access_token is None:
            self.error = mark_safe(_(
                'Please <a href="%s">attach Yandex account and choose Yandex Metrika counter</a> to start using '
                'widget') % reverse(
                'jet-dashboard:update_module', kwargs={'pk': self.model.pk}))
            return False
        elif self.counter is None:
            self.error = mark_safe(
                _('Please <a href="%s">select Yandex Metrika counter</a> to start using widget') % reverse(
                    'jet-dashboard:update_module', kwargs={'pk': self.model.pk}))
            return False
        else:
            return True

    def api_stat_traffic_summary(self, group=None):
        if self.counter_attached():
            date1 = datetime.datetime.now() - datetime.timedelta(days=self.period)
            date2 = datetime.datetime.now()

            client = YandexMetrikaClient(self.access_token)
            result, exception = client.api_stat_traffic_summary(self.counter, date1, date2, group)

            if exception is not None:
                error = _('API request failed.')
                if isinstance(exception, HTTPError) and exception.code == 403:
                    error += _(' Try to <a href="%s">revoke and grant access</a> again') % reverse(
                        'jet-dashboard:update_module', kwargs={'pk': self.model.pk})
                self.error = mark_safe(error)
            else:
                return result


class YandexMetrikaVisitorsTotals(YandexMetrikaBase):
    """
    Yandex Metrika widget that shows total number of visitors, visits and viewers for a particular period of time.
    Period may be following: Today, Last week, Last month, Last quarter, Last year
    """

    title = _('Yandex Metrika visitors totals')
    template = 'jet.dashboard/modules/yandex_metrika_visitors_totals.html'

    #: Which period should be displayed. Allowed values - integer of days
    period = None

    def __init__(self, title=None, period=None, **kwargs):
        kwargs.update({'period': period})
        super(YandexMetrikaVisitorsTotals, self).__init__(title, **kwargs)

    def init_with_context(self, context):
        result = self.api_stat_traffic_summary()

        if result is not None:
            try:
                self.children.append({'title': _('visitors'), 'value': result['totals']['visitors']})
                self.children.append({'title': _('visits'), 'value': result['totals']['visits']})
                self.children.append({'title': _('views'), 'value': result['totals']['page_views']})
            except KeyError:
                self.error = _('Bad server response')


class YandexMetrikaVisitorsChart(YandexMetrikaBase):
    """
    Yandex Metrika widget that shows visitors/visits/viewer chart for a particular period of time.
    Data is grouped by day, week or month
    Period may be following: Today, Last week, Last month, Last quarter, Last year
    """

    title = _('Yandex Metrika visitors chart')
    template = 'jet.dashboard/modules/yandex_metrika_visitors_chart.html'
    style = 'overflow-x: auto;'

    #: Which period should be displayed. Allowed values - integer of days
    period = None

    #: What data should be shown. Possible values: ``visitors``, ``visits``, ``page_views``
    show = None

    #: Sets grouping of data. Possible values: ``day``, ``week``, ``month``
    group = None
    settings_form = YandexMetrikaChartSettingsForm

    class Media:
        js = ('jet.dashboard/vendor/chart.js/Chart.min.js', 'jet.dashboard/modules/yandex_metrika.js')

    def __init__(self, title=None, period=None, show=None, group=None, **kwargs):
        kwargs.update({'period': period, 'show': show, 'group': group})
        super(YandexMetrikaVisitorsChart, self).__init__(title, **kwargs)

    def settings_dict(self):
        settings = super(YandexMetrikaVisitorsChart, self).settings_dict()
        settings['show'] = self.show
        settings['group'] = self.group
        return settings

    def load_settings(self, settings):
        super(YandexMetrikaVisitorsChart, self).load_settings(settings)
        self.show = settings.get('show')
        self.group = settings.get('group')

    def init_with_context(self, context):
        result = self.api_stat_traffic_summary(self.group)

        if result is not None:
            try:
                for data in result['data']:
                    date = datetime.datetime.strptime(data['date'], '%Y%m%d')
                    key = self.show if self.show is not None else 'visitors'
                    self.children.append((date, data[key]))
            except KeyError:
                self.error = _('Bad server response')


class YandexMetrikaPeriodVisitors(YandexMetrikaBase):
    """
    Yandex Metrika widget that shows visitors, visits and viewers for a particular period of time.
    Data is grouped by day, week or month
    Period may be following: Today, Last week, Last month, Last quarter, Last year
    """

    title = _('Yandex Metrika period visitors')
    template = 'jet.dashboard/modules/yandex_metrika_period_visitors.html'

    #: Which period should be displayed. Allowed values - integer of days
    period = None

    #: Sets grouping of data. Possible values: ``day``, ``week``, ``month``
    group = None
    contrast = False
    settings_form = YandexMetrikaPeriodVisitorsSettingsForm

    def __init__(self, title=None, period=None, group=None, **kwargs):
        kwargs.update({'period': period, 'group': group})
        super(YandexMetrikaPeriodVisitors, self).__init__(title, **kwargs)

    def settings_dict(self):
        settings = super(YandexMetrikaPeriodVisitors, self).settings_dict()
        settings['group'] = self.group
        return settings

    def load_settings(self, settings):
        super(YandexMetrikaPeriodVisitors, self).load_settings(settings)
        self.group = settings.get('group')

    def init_with_context(self, context):
        result = self.api_stat_traffic_summary(self.group)

        if result is not None:
            try:
                for data in reversed(result['data']):
                    date = datetime.datetime.strptime(data['date'], '%Y%m%d')
                    date = self.format_grouped_date(date, self.group)
                    self.children.append((date, data))
            except KeyError:
                self.error = _('Bad server response')



from django.core.management import BaseCommand, CommandError

from jet.sidebar import get_original_menu_items


class Command(BaseCommand):
    help = 'Generates example of JET custom apps setting'
    item_order = 0

    def handle(self, *args, **options):
        if args:
            raise CommandError("Command doesn't accept any arguments")
        return self.handle_noargs(**options)

    def handle_noargs(self, **options):
        class User:
            is_active = True
            is_staff = True
            is_superuser = True

            def has_module_perms(self, app):
                return True

            def has_perm(self, object):
                return True

        class Request:
            user = User()

        app_list = get_original_menu_items({
            'request': Request(),
            'user': None
        })

        self.stdout.write("# Add this to your settings.py to customize applications and models list")
        self.stdout.write("JET_SIDE_MENU_ITEMS = [")
        for app in app_list:
            self.stdout.write("    {'app_label': '%s', 'items': [" % (app.app_label,))
            for model in app.models:
                self.stdout.write("        {'name': '%s'}," % (model.name,))
            self.stdout.write("    ]},")
        self.stdout.write("]")



import operator
from functools import reduce

from django import forms
from django.apps.registry import apps
from django.core.exceptions import ValidationError
from django.db.models import Q

from jet.models import Bookmark, PinnedApplication
from jet.utils import user_is_authenticated


class AddBookmarkForm(forms.ModelForm):
    def __init__(self, request, *args, **kwargs):
        self.request = request
        super(AddBookmarkForm, self).__init__(*args, **kwargs)

    class Meta:
        model = Bookmark
        fields = ['url', 'title']

    def clean(self):
        data = super(AddBookmarkForm, self).clean()
        if not user_is_authenticated(self.request.user) or not self.request.user.is_staff:
            raise ValidationError('error')
        if not self.request.user.has_perm('jet.change_bookmark'):
            raise ValidationError('error')
        return data

    def save(self, commit=True):
        self.instance.user = self.request.user.pk
        return super(AddBookmarkForm, self).save(commit)


class RemoveBookmarkForm(forms.ModelForm):
    def __init__(self, request, *args, **kwargs):
        self.request = request
        super(RemoveBookmarkForm, self).__init__(*args, **kwargs)

    class Meta:
        model = Bookmark
        fields = []

    def clean(self):
        data = super(RemoveBookmarkForm, self).clean()
        if not user_is_authenticated(self.request.user) or not self.request.user.is_staff:
            raise ValidationError('error')
        if self.instance.user != self.request.user.pk:
            raise ValidationError('error')
        return data

    def save(self, commit=True):
        if commit:
            self.instance.delete()


class ToggleApplicationPinForm(forms.ModelForm):
    def __init__(self, request, *args, **kwargs):
        self.request = request
        super(ToggleApplicationPinForm, self).__init__(*args, **kwargs)

    class Meta:
        model = PinnedApplication
        fields = ['app_label']

    def clean(self):
        data = super(ToggleApplicationPinForm, self).clean()
        if not user_is_authenticated(self.request.user) or not self.request.user.is_staff:
            raise ValidationError('error')
        return data

    def save(self, commit=True):
        if commit:
            try:
                pinned_app = PinnedApplication.objects.get(
                    app_label=self.cleaned_data['app_label'],
                    user=self.request.user.pk
                )
                pinned_app.delete()
                return False
            except PinnedApplication.DoesNotExist:
                PinnedApplication.objects.create(
                    app_label=self.cleaned_data['app_label'],
                    user=self.request.user.pk
                )
                return True


class ModelLookupForm(forms.Form):
    app_label = forms.CharField()
    model = forms.CharField()
    q = forms.CharField(required=False)
    page = forms.IntegerField(required=False)
    page_size = forms.IntegerField(required=False, min_value=1, max_value=1000)
    object_id = forms.IntegerField(required=False)
    lookup_kwarg = forms.CharField(required=False)
    lookup_params = forms.CharField(required=False)
    model_cls = None

    def __init__(self, request, *args, **kwargs):
        self.request = request
        super(ModelLookupForm, self).__init__(*args, **kwargs)

    def clean(self):
        data = super(ModelLookupForm, self).clean()

        if not user_is_authenticated(self.request.user) or not self.request.user.is_staff:
            raise ValidationError('user is not authenticated or is not staff')

        try:
            self.model_cls = apps.get_model(data['app_label'], data['model'])
        except Exception as e:
            raise ValidationError(e)

        app = data['app_label']
        model = self.model_cls._meta.model_name
        permissions = (f"{app}.view_{model}", f"{app}.change_{model}")
        if not any(map(self.request.user.has_perm, permissions)):
            raise ValidationError("user doesn't have permissions")

        return data

    def get_queryset(self):
        qs = self.model_cls.objects.all()
        if self.cleaned_data['q']:
            if getattr(self.model_cls, 'autocomplete_search_fields', None):
                search_fields = self.model_cls.autocomplete_search_fields()
                filter_data = [Q((field + '__icontains', self.cleaned_data['q'])) for field in search_fields]
                qs = qs.filter(reduce(operator.or_, filter_data)).distinct()
            else:
                qs = qs.none()
        return qs



import json
import os
from urllib.parse import parse_qsl

from django import template
from django.contrib.admin.widgets import RelatedFieldWidgetWrapper
from django.forms import (
    BoundField, CheckboxInput, Field, ModelChoiceField, ModelMultipleChoiceField, Select,
    SelectMultiple,
)
from django.urls import reverse
from django.utils.encoding import smart_str
from django.utils.formats import get_format
from django.utils.safestring import mark_safe

from jet import __version__, settings
from jet.models import Bookmark
from jet.sidebar import Sidebar, get_menu_items
from jet.utils import (
    format_widget_data, get_admin_site, get_model_instance_label,
    get_model_queryset, get_possible_language_codes, import_value,
)


register = template.Library()
assignment_tag = register.assignment_tag if hasattr(register, 'assignment_tag') else register.simple_tag
Sidebar = import_value(settings.JET_SIDE_MENU_CLS)


@assignment_tag
def jet_get_date_format():
    return get_format('DATE_INPUT_FORMATS')[0]


@assignment_tag
def jet_get_time_format():
    return get_format('TIME_INPUT_FORMATS')[0]


@assignment_tag
def jet_get_datetime_format():
    return get_format('DATETIME_INPUT_FORMATS')[0]


@assignment_tag(takes_context=True)
def jet_get_menu(context):
    return get_menu_items(context)


@assignment_tag
def jet_get_bookmarks(user):
    if user is None:
        return None
    return Bookmark.objects.filter(user=user.pk)


@register.filter
def jet_is_checkbox(field):
    return field.field.widget.__class__.__name__ == CheckboxInput().__class__.__name__


@register.filter
def jet_select2_lookups(field: BoundField):
    form_field: Field = getattr(field, 'field', None)
    if not (
        form_field and
        (isinstance(form_field, ModelChoiceField) or
         isinstance(form_field, ModelMultipleChoiceField))
    ):
        return field

    qs = form_field.queryset
    model = qs.model

    if not (
        getattr(model, 'autocomplete_search_fields', None) and
        getattr(form_field, 'autocomplete', True)
    ):
        return field

    choices = []
    app_label = model._meta.app_label
    model_name = model._meta.object_name
    url = getattr(form_field, 'url', reverse('jet:model_lookup'))

    data = getattr(form_field.widget, 'data', {})
    data['blank'] = not form_field.required

    attrs = {
        'class': 'ajax',
        'data-app-label': app_label,
        'data-model': model_name,
        'data-ajax--url': url,
        **format_widget_data(data),
    }

    initial_value = field.value()

    if isinstance(form_field, ModelMultipleChoiceField):
        if initial_value:
            initial_objects = model.objects.filter(pk__in=initial_value)
            choices.extend(
                [(initial_object.pk, get_model_instance_label(initial_object))
                 for initial_object in initial_objects]
            )

        if isinstance(form_field.widget, RelatedFieldWidgetWrapper):
            form_field.widget.widget = SelectMultiple(attrs)
        else:
            form_field.widget = SelectMultiple(attrs)
        form_field.choices = choices
    elif isinstance(form_field, ModelChoiceField):
        if initial_value:
            try:
                initial_object = model.objects.get(pk=initial_value)
                attrs['data-object-id'] = initial_value
                choices.append((initial_object.pk, get_model_instance_label(initial_object)))
            except model.DoesNotExist:
                pass

        if isinstance(form_field.widget, RelatedFieldWidgetWrapper):
            form_field.widget.widget = Select(attrs)
        else:
            form_field.widget = Select(attrs)
        form_field.choices = choices

    return field


@assignment_tag(takes_context=True)
def jet_get_current_theme(context):
    if 'request' in context and 'JET_THEME' in context['request'].COOKIES:
        theme = context['request'].COOKIES['JET_THEME']
        if isinstance(settings.JET_THEMES, list) and len(settings.JET_THEMES) > 0:
            for conf_theme in settings.JET_THEMES:
                if isinstance(conf_theme, dict) and conf_theme.get('theme') == theme:
                    return theme
    return settings.JET_DEFAULT_THEME


@assignment_tag
def jet_get_themes():
    return settings.JET_THEMES


@assignment_tag
def jet_get_current_version():
    return __version__


@register.filter
def jet_append_version(url):
    if '?' in url:
        return '%s&v=%s' % (url, __version__)
    else:
        return '%s?v=%s' % (url, __version__)


@assignment_tag
def jet_get_side_menu_compact():
    return settings.JET_SIDE_MENU_COMPACT


@assignment_tag
def jet_change_form_sibling_links_enabled():
    return settings.JET_CHANGE_FORM_SIBLING_LINKS


def jet_sibling_object(context, next):
    original = context.get('original')

    if not original:
        return

    model = type(original)
    preserved_filters_plain = context.get('preserved_filters', '')
    preserved_filters = dict(parse_qsl(preserved_filters_plain))
    admin_site = get_admin_site(context)

    if admin_site is None:
        return

    request = context.get('request')
    queryset = get_model_queryset(admin_site, model, request, preserved_filters=preserved_filters)

    if queryset is None:
        return

    sibling_object = None
    object_pks = list(queryset.values_list('pk', flat=True))

    try:
        index = object_pks.index(original.pk)
        sibling_index = index + 1 if next else index - 1
        exists = sibling_index < len(object_pks) if next else sibling_index >= 0
        sibling_object = queryset.get(pk=object_pks[sibling_index]) if exists else None
    except ValueError:
        pass

    if sibling_object is None:
        return

    url = reverse('%s:%s_%s_change' % (
        admin_site.name,
        model._meta.app_label,
        model._meta.model_name
    ), args=(sibling_object.pk,))

    if preserved_filters_plain != '':
        url += '?' + preserved_filters_plain

    return {
        'label': smart_str(sibling_object),
        'url': url
    }


@assignment_tag(takes_context=True)
def jet_previous_object(context):
    return jet_sibling_object(context, False)


@assignment_tag(takes_context=True)
def jet_next_object(context):
    return jet_sibling_object(context, True)


@assignment_tag(takes_context=True)
def jet_popup_response_data(context):
    if context.get('popup_response_data'):
        return context['popup_response_data']

    return json.dumps({
        'action': context.get('action'),
        'value': context.get('value') or context.get('pk_value'),
        'obj': smart_str(context.get('obj')),
        'new_value': context.get('new_value')
    })


@assignment_tag(takes_context=True)
def jet_delete_confirmation_context(context):
    if context.get('deletable_objects') is None and context.get('deleted_objects') is None:
        return ''
    return mark_safe('<div class="delete-confirmation-marker"></div>')


@assignment_tag
def jet_static_translation_urls():
    language_codes = get_possible_language_codes()

    urls = []
    url_templates = [
        'jet/js/i18n/jquery-ui/datepicker-__LANGUAGE_CODE__.js',
        'jet/js/i18n/jquery-ui-timepicker/jquery.ui.timepicker-__LANGUAGE_CODE__.js',
        'jet/js/i18n/select2/__LANGUAGE_CODE__.js'
    ]

    static_dir = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'static')

    for tpl in url_templates:
        for language_code in language_codes:
            url = tpl.replace('__LANGUAGE_CODE__', language_code)
            path = os.path.join(static_dir, url)

            if os.path.exists(path):
                urls.append(url)
                break

    return urls


@register.simple_tag(takes_context=True)
def jet_render_sidebar(context):
    return Sidebar().render(context)



from typing import Dict, List

from django.utils.text import capfirst, slugify
from django.utils.translation import gettext_lazy as _
from pydantic import BaseModel

from jet import settings
from jet.models import PinnedApplication
from jet.utils import get_admin_site, get_app_list, user_is_authenticated


class MenuItem(BaseModel):
    name: str = None
    label: str = None
    url: str = None
    url_blank = False
    has_perms = False
    object_name: str = None
    current = False


class MenuSection(BaseModel):
    app_label: str = None
    label: str = None
    url: str = None
    url_blank = False
    has_perms = False
    items: List[MenuItem] = []
    pinned = False
    current = False
    models: List[MenuItem] = []


class ItemConfig(BaseModel):
    """
    name - model name (can be either MODEL_NAME or APP_LABEL.MODEL_NAME)
    label - item text label
    url - custom url
    url_blank - open url in new table
    permissions - list of required permissions to display item
    """
    name: str = None
    label: str = None
    url: str = None
    url_blank: bool = None
    permissions: List[str] = None


class AppConfig(BaseModel):
    """
    app_label - application name
    label - application text label
    items - list of children items
    url - custom url
    url_blank - open url in new table
    permissions - list of required permissions to display item
    """
    app_label: str = None
    label: str = None
    items: List[ItemConfig] = None
    url: str = None
    url_blank: bool = None
    permissions: List[str] = None


def get_original_menu_items(context) -> List[MenuSection]:
    if context.get('user') and user_is_authenticated(context['user']):
        pinned_apps = PinnedApplication.objects.filter(user=context['user'].pk).values_list('app_label', flat=True)
    else:
        pinned_apps = []
    return [
        MenuSection(
            app_label=app['app_label'],
            url=app['app_url'],
            label=str(app.get('name', capfirst(_(app['app_label'])))),
            has_perms=app.get('has_module_perms', False),
            pinned=app['app_label'] in pinned_apps,
            models=[
                MenuItem(
                    url=model.get('admin_url'),
                    name=model['model_name'],
                    object_name=model['object_name'],
                    label=str(model.get('name', model['object_name'])),
                    has_perms=any(model.get('perms', {}).values()),
                )
                for model in app['models']
            ]
        )
        for app in get_app_list(context)
    ]

    # return map(lambda app: {
    #     'app_label': app['app_label'],
    #     'url': app['app_url'],
    #     'url_blank': False,
    #     'label': app.get('name', capfirst(_(app['app_label']))),
    #     'has_perms': app.get('has_module_perms', False),
    #     'models': list(map(lambda model: {
    #         'url': model.get('admin_url'),
    #         'url_blank': False,
    #         'name': model['model_name'],
    #         'object_name': model['object_name'],
    #         'label': model.get('name', model['object_name']),
    #         'has_perms': any(model.get('perms', {}).values()),
    #     }, app['models'])),
    #     'pinned': app['app_label'] in pinned_apps,
    #     'custom': False
    # }, app_list)


class MenuBuilder:
    def __init__(self, context):
        self.context = context

    def get_original_menu_item(self, name: str, app_label: str, original_apps):
        parts = name.split('.', 2)
        if len(parts) > 1:
            app_label, name = parts
        if app_label in original_apps:
            for model in original_apps[app_label].models:
                if model.name == name:
                    return model

    def get_menu_item(self, config: ItemConfig, app_label: str, original_apps: Dict[str, MenuSection]):
        item = MenuItem(has_perms=True)
        if config.name:
            original_item = self.get_original_menu_item(config.name, app_label, original_apps)
            if original_item:
                item = original_item.copy()
            else:
                item.has_perms = False
        if config.label is not None:
            item.label = config.label
        if config.url is not None:
            item.url = config.url
        if config.url_blank is not None:
            item.url_blank = config.url_blank
        if config.permissions is not None:
            item.has_perms = item.has_perms and self.context['user'].has_perms(config.permissions)
        return item

    def get_menu_section(self, config: AppConfig, original_apps: Dict[str, MenuSection], pinned_apps: List[str]):
        section = MenuSection(app_label=config.app_label)

        if not config.app_label and not config.label:
            raise ValueError("Custom menu items should at least have 'label' or 'app_label' key")

        # custom section viewable for everyone
        if not config.app_label:
            section.app_label = 'custom_%s' % slugify(config.label, allow_unicode=True)
            section.has_perms = True
        # original section in has_perms=True
        elif section.app_label in original_apps:
            section = original_apps[section.app_label].copy()
        # non-custom, not original - no permissions
        else:
            section.has_perms = False

        if config.label is not None:
            section.label = config.label
        if config.url is not None:
            section.url = config.url
        if config.url_blank is not None:
            section.url_blank = config.url_blank
        if config.permissions is not None:
            section.has_perms = section.has_perms and self.context['user'].has_perms(config.permissions)
        if config.items is not None:
            section.items = [
                self.get_menu_item(item, section.app_label, original_apps)
                for item in config.items
            ]
        section.pinned = section.app_label in pinned_apps
        return section

    def build_from_config(self, custom_apps, original_apps):
        if isinstance(custom_apps, dict):
            admin_site = get_admin_site(self.context)
            custom_apps = custom_apps.get(admin_site.name, [])

        custom_apps = [AppConfig.parse_obj(app) for app in custom_apps]

        pinned_apps = PinnedApplication.objects.filter(user=self.context['user'].pk)
        pinned_apps = pinned_apps.values_list('app_label', flat=True)
        return [
            self.get_menu_section(app, original_apps, pinned_apps)
            for app in custom_apps
        ]

    def build(self):
        original_apps = {
            app.app_label: app
            for app in get_original_menu_items(self.context)
        }
        custom_apps = settings.JET_SIDE_MENU_ITEMS
        if custom_apps not in (None, False):
            app_list = self.build_from_config(custom_apps, original_apps)
        else:
            app_list = list(original_apps.values())
            for app in app_list:
                app.items = app.models
        self.setup_current_app(app_list)
        return app_list

    def setup_current_app(self, apps: List[MenuSection]):
        for app in apps:
            for model in app.items:
                if model.url and self.context['request'].path.startswith(model.url):
                    model.current = True
                    return
            if app.url and self.context['request'].path.startswith(app.url):
                app.current = True
                return


def get_menu_items(context) -> List[MenuSection]:
    b = MenuBuilder(context)
    return b.build()



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class JetDashboardConfig(AppConfig):
    name = 'jet.dashboard'
    label = 'jet.dashboard'
    verbose_name = _('Jet Dashboard')

    # noinspection PyUnresolvedReferences
    def ready(self):
        from . import checks



from django import forms
from django.contrib.admin.models import LogEntry
from django.db.models import Q
from django.utils.translation import gettext_lazy as _

from .base import DashboardModule


class RecentActionsSettingsForm(forms.Form):
    limit = forms.IntegerField(label=_('Items limit'), min_value=1)


class RecentActions(DashboardModule):
    """
    Display list of most recent admin actions with following information:
    entity name, type of action, author, date

    Usage example:

    .. code-block:: python

        from django.utils.translation import gettext_lazy as _
        from jet.dashboard import modules
        from jet.dashboard.dashboard import Dashboard, AppIndexDashboard


        class CustomIndexDashboard(Dashboard):
            columns = 3

            def init_with_context(self, context):
                self.children.append(modules.RecentActions(
                    _('Recent Actions'),
                    10,
                    column=0,
                    order=0
                ))

    """

    title = _('Recent Actions')
    template = 'jet.dashboard/modules/recent_actions.html'

    #: Number if entries to be shown (may be changed by each user personally).
    limit = 10

    #: Specify actions of which models should be displayed. ``include_list`` is an array of string
    #: formatted as ``app_label.model``. Also its possible to specify all application models
    #: with * sign (e.g. ``auth.*``).
    include_list = None

    #: Specify actions of which models should NOT be displayed. ``exclude_list`` is an array of string
    #: formatted as ``app_label.model``. Also its possible to specify all application models
    #: with * sign (e.g. ``auth.*``).
    exclude_list = None
    settings_form = RecentActionsSettingsForm
    user = None

    def __init__(self, title=None, limit=10, **kwargs):
        kwargs.update({'limit': limit})
        super(RecentActions, self).__init__(title, **kwargs)

    def settings_dict(self):
        return {
            'limit': self.limit,
            'include_list': self.include_list,
            'exclude_list': self.exclude_list,
            'user': self.user
        }

    def load_settings(self, settings):
        self.limit = settings.get('limit', self.limit)
        self.include_list = settings.get('include_list')
        self.exclude_list = settings.get('exclude_list')
        self.user = settings.get('user', None)

    def init_with_context(self, context):
        def get_qset(list):
            qset = None
            for contenttype in list:
                try:
                    app_label, model = contenttype.split('.')

                    if model == '*':
                        current_qset = Q(
                            content_type__app_label=app_label
                        )
                    else:
                        current_qset = Q(
                            content_type__app_label=app_label,
                            content_type__model=model
                        )
                except:
                    raise ValueError('Invalid contenttype: "%s"' % contenttype)

                if qset is None:
                    qset = current_qset
                else:
                    qset = qset | current_qset
            return qset

        qs = LogEntry.objects

        if self.user:
            qs = qs.filter(
                user__pk=int(self.user)
            )

        if self.include_list:
            qs = qs.filter(get_qset(self.include_list))
        if self.exclude_list:
            qs = qs.exclude(get_qset(self.exclude_list))

        self.children = qs.select_related('content_type', 'user')[:int(self.limit)]



from django.urls import path
from django.views.i18n import JavaScriptCatalog

from jet.views import add_bookmark_view, model_lookup_view, remove_bookmark_view, toggle_application_pin_view


app_name = 'jet'

urlpatterns = [
    path(
        'add_bookmark/',
        add_bookmark_view,
        name='add_bookmark'
    ),
    path(
        'remove_bookmark/',
        remove_bookmark_view,
        name='remove_bookmark'
    ),
    path(
        'toggle_application_pin/',
        toggle_application_pin_view,
        name='toggle_application_pin'
    ),
    path(
        'model_lookup/',
        model_lookup_view,
        name='model_lookup'
    ),
    path(
        'jsi18n/',
        JavaScriptCatalog.as_view(),
        {'packages': 'django.contrib.admin+jet'},
        name='jsi18n'
    ),
]



from django.urls import include, path, re_path
from django.views.i18n import JavaScriptCatalog

from jet.dashboard import settings, views


app_name = 'dashboard'

urlpatterns = [
    re_path(
        r'^module/(?P<pk>\d+)/$',
        views.UpdateDashboardModuleView.as_view(),
        name='update_module'
    ),
    path(
        'update_dashboard_modules/',
        views.update_dashboard_modules_view,
        name='update_dashboard_modules'
    ),
    path(
        'add_user_dashboard_module/',
        views.add_user_dashboard_module_view,
        name='add_user_dashboard_module'
    ),
    path(
        'update_dashboard_module_collapse/',
        views.update_dashboard_module_collapse_view,
        name='update_dashboard_module_collapse'
    ),
    path(
        'remove_dashboard_module/',
        views.remove_dashboard_module_view,
        name='remove_dashboard_module'
    ),
    re_path(
        r'^load_dashboard_module/(?P<pk>\d+)/$',
        views.load_dashboard_module_view,
        name='load_dashboard_module'
    ),
    path(
        'reset_dashboard/',
        views.reset_dashboard_view,
        name='reset_dashboard'
    ),
    path(
        'jsi18n/',
        JavaScriptCatalog.as_view(),
        {'packages': 'jet'},
        name='jsi18n'
    ),
]

if settings.JET_MODULE_GOOGLE_ANALYTICS_CLIENT_SECRETS_FILE:
    urlpatterns.append(path('', include('jet.dashboard.modules.google_analytics.urls')))

if (
    settings.JET_MODULE_YANDEX_METRIKA_CLIENT_ID and
    settings.JET_MODULE_YANDEX_METRIKA_CLIENT_SECRET
):
    urlpatterns.append(path('', include('jet.dashboard.modules.yandex_metrika.urls')))



from django.contrib import messages
from django.http import HttpResponse
from django.shortcuts import redirect
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from httplib2 import ServerNotFoundError
from oauth2client.client import FlowExchangeError

from jet.dashboard.models import UserDashboardModule
from .module import GoogleAnalyticsClient, ModuleCredentialStorage


def google_analytics_grant_view(request, pk):
    redirect_uri = request.build_absolute_uri(reverse('jet-dashboard:google-analytics-callback'))
    client = GoogleAnalyticsClient(redirect_uri=redirect_uri)
    return redirect(client.get_oauth_authorize_url(pk))


def google_analytics_revoke_view(request, pk):
    try:
        module = UserDashboardModule.objects.get(pk=pk)
        ModuleCredentialStorage(module).delete()
        return redirect(reverse('jet-dashboard:update_module', kwargs={'pk': module.pk}))
    except UserDashboardModule.DoesNotExist:
        return HttpResponse(_('Module not found'))


def google_analytics_callback_view(request):
    module = None

    try:
        state = request.GET['state']
        module = UserDashboardModule.objects.get(pk=state)

        redirect_uri = request.build_absolute_uri(reverse('jet-dashboard:google-analytics-callback'))
        client = GoogleAnalyticsClient(redirect_uri=redirect_uri)
        client.set_credential_from_request(request)

        ModuleCredentialStorage(module).put(client.credential)
    except (FlowExchangeError, ValueError, ServerNotFoundError):
        messages.error(request, _('API request failed.'))
    except KeyError:
        return HttpResponse(_('Bad arguments'))
    except UserDashboardModule.DoesNotExist:
        return HttpResponse(_('Module not found'))

    return redirect(reverse('jet-dashboard:update_module', kwargs={'pk': module.pk}))



import datetime
import json

import httplib2
from django import forms
from django.forms import Widget
from django.forms.utils import flatatt
from django.urls import reverse
from django.utils import formats
from django.utils.encoding import force_text
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.text import capfirst
from django.utils.translation import gettext_lazy as _
from googleapiclient.discovery import build
from oauth2client.client import AccessTokenRefreshError, OAuth2Credentials, Storage, flow_from_clientsecrets

from jet.dashboard.settings import JET_MODULE_GOOGLE_ANALYTICS_CLIENT_SECRETS_FILE
from ..base import DashboardModule


class ModuleCredentialStorage(Storage):
    def __init__(self, module):
        super(ModuleCredentialStorage, self).__init__()
        self.module = module

    def locked_get(self):
        pass

    def locked_put(self, credentials):
        pass

    def locked_delete(self):
        pass

    def get(self):
        try:
            settings = json.loads(self.module.settings)
            credential = settings['credential']
            return OAuth2Credentials.from_json(credential)
        except (ValueError, KeyError):
            return None

    def put(self, credentials):
        self.module.update_settings({'credential': credentials.to_json()})

    def delete(self):
        self.module.pop_settings(('credential',))


class GoogleAnalyticsClient:
    credential = None
    analytics_service = None

    def __init__(self, storage=None, redirect_uri=None):
        self.FLOW = flow_from_clientsecrets(
            JET_MODULE_GOOGLE_ANALYTICS_CLIENT_SECRETS_FILE,
            scope='https://www.googleapis.com/auth/analytics.readonly',
            redirect_uri=redirect_uri,
            prompt='consent'
        )

        if storage is not None:
            credential = storage.get()
            credential.set_store(storage)
            self.set_credential(credential)

    def get_oauth_authorize_url(self, state=''):
        self.FLOW.params['state'] = state
        authorize_url = self.FLOW.step1_get_authorize_url()
        return authorize_url

    def set_credential(self, credential):
        self.credential = credential
        self.set_analytics_service(self.credential)

    def set_credential_from_request(self, request):
        self.set_credential(self.FLOW.step2_exchange(request.GET))

    def set_analytics_service(self, credential):
        http = httplib2.Http()
        http = credential.authorize(http)
        self.analytics_service = build('analytics', 'v3', http=http)

    def api_profiles(self):
        if self.analytics_service is None:
            return None, None

        try:
            profiles = self.analytics_service.management().profiles().list(
                accountId='~all',
                webPropertyId='~all'
            ).execute()

            return profiles['items'], None
        except (TypeError, KeyError) as e:
            return None, e

    def api_ga(self, profile_id, date1, date2, group=None):
        if self.analytics_service is None:
            return None, None

        if group == 'day':
            dimensions = 'ga:date'
        elif group == 'week':
            dimensions = 'ga:year,ga:week'
        elif group == 'month':
            dimensions = 'ga:year,ga:month'
        else:
            dimensions = ''

        try:
            data = self.analytics_service.data().ga().get(
                ids='ga:' + profile_id,
                start_date=date1.strftime('%Y-%m-%d'),
                end_date=date2.strftime('%Y-%m-%d'),
                metrics='ga:users,ga:sessions,ga:pageviews',
                dimensions=dimensions
            ).execute()

            return data, None
        except TypeError as e:
            return None, e


class CredentialWidget(Widget):
    module = None

    def render(self, name, value, attrs=None, rendered=None):
        if value and len(value) > 0:
            link = '<a href="%s">%s</a>' % (
                reverse('jet-dashboard:google-analytics-revoke', kwargs={'pk': self.module.model.pk}),
                force_text(_('Revoke access'))
            )
        else:
            link = '<a href="%s">%s</a>' % (
                reverse('jet-dashboard:google-analytics-grant', kwargs={'pk': self.module.model.pk}),
                force_text(_('Grant access'))
            )

        attrs = self.build_attrs({
            'type': 'hidden',
            'name': 'credential',
        })
        attrs['value'] = force_text(value) if value else ''

        return format_html('%s<input{} />' % link, flatatt(attrs))


class GoogleAnalyticsSettingsForm(forms.Form):
    credential = forms.CharField(label=_('Access'), widget=CredentialWidget)
    counter = forms.ChoiceField(label=_('Counter'))
    period = forms.ChoiceField(label=_('Statistics period'), choices=(
        (0, _('Today')),
        (6, _('Last week')),
        (30, _('Last month')),
        (31 * 3 - 1, _('Last quarter')),
        (364, _('Last year')),
    ))

    def set_module(self, module):
        self.fields['credential'].widget.module = module
        self.set_counter_choices(module)

    def set_counter_choices(self, module):
        counters = module.counters()
        if counters is not None:
            self.fields['counter'].choices = (('', '-- %s --' % force_text(_('none'))),)
            self.fields['counter'].choices.extend(map(lambda x: (x['id'], x['websiteUrl']), counters))
        else:
            label = force_text(_('grant access first')) if module.credential is None else force_text(
                _('counters loading failed'))
            self.fields['counter'].choices = (('', '-- %s -- ' % label),)


class GoogleAnalyticsChartSettingsForm(GoogleAnalyticsSettingsForm):
    show = forms.ChoiceField(label=_('Show'), choices=(
        ('ga:users', capfirst(_('users'))),
        ('ga:sessions', capfirst(_('sessions'))),
        ('ga:pageviews', capfirst(_('views'))),
    ))
    group = forms.ChoiceField(label=_('Group'), choices=(
        ('day', _('By day')),
        ('week', _('By week')),
        ('month', _('By month')),
    ))


class GoogleAnalyticsPeriodVisitorsSettingsForm(GoogleAnalyticsSettingsForm):
    group = forms.ChoiceField(label=_('Group'), choices=(
        ('day', _('By day')),
        ('week', _('By week')),
        ('month', _('By month')),
    ))


class GoogleAnalyticsBase(DashboardModule):
    settings_form = GoogleAnalyticsSettingsForm
    ajax_load = True
    contrast = True
    period = None
    credential = None
    counter = None
    error = None
    storage = None

    def __init__(self, title=None, period=None, **kwargs):
        kwargs.update({'period': period})
        super(GoogleAnalyticsBase, self).__init__(title, **kwargs)

    def settings_dict(self):
        return {
            'period': self.period,
            'credential': self.credential,
            'counter': self.counter
        }

    def load_settings(self, settings):
        try:
            self.period = int(settings.get('period'))
        except TypeError:
            self.period = 0
        self.credential = settings.get('credential')
        self.storage = ModuleCredentialStorage(self.model)
        self.counter = settings.get('counter')

    def init_with_context(self, context):
        raise NotImplementedError('subclasses of GoogleAnalytics must provide a init_with_context() method')

    def counters(self):
        try:
            client = GoogleAnalyticsClient(self.storage)
            profiles, exception = client.api_profiles()
            return profiles
        except Exception:
            return None

    def get_grouped_date(self, data, group):
        if group == 'week':
            date = datetime.datetime.strptime(
                '%s-%s-%s' % (data['ga_year'], data['ga_week'], '0'),
                '%Y-%W-%w'
            )
        elif group == 'month':
            date = datetime.datetime.strptime(data['ga_year'] + data['ga_month'], '%Y%m')
        else:
            date = datetime.datetime.strptime(data['ga_date'], '%Y%m%d')
        return date

    def format_grouped_date(self, data, group):
        date = self.get_grouped_date(data, group)

        if group == 'week':
            date = u'%s — %s' % (
                (date - datetime.timedelta(days=6)).strftime('%d.%m'),
                date.strftime('%d.%m')
            )
        elif group == 'month':
            date = date.strftime('%b, %Y')
        else:
            date = formats.date_format(date, 'DATE_FORMAT')
        return date

    def counter_attached(self):
        if self.credential is None:
            self.error = mark_safe(_(
                'Please <a href="%s">attach Google account and choose Google Analytics counter</a> to start using '
                'widget') % reverse(
                'jet-dashboard:update_module', kwargs={'pk': self.model.pk}))
            return False
        elif self.counter is None:
            self.error = mark_safe(
                _('Please <a href="%s">select Google Analytics counter</a> to start using widget') % reverse(
                    'jet-dashboard:update_module', kwargs={'pk': self.model.pk}))
            return False
        else:
            return True

    def api_ga(self, group=None):
        if self.counter_attached():
            date1 = datetime.datetime.now() - datetime.timedelta(days=self.period)
            date2 = datetime.datetime.now()

            try:
                client = GoogleAnalyticsClient(self.storage)
                result, exception = client.api_ga(self.counter, date1, date2, group)

                if exception is not None:
                    raise exception

                return result
            except Exception as e:
                error = _('API request failed.')
                if isinstance(e, AccessTokenRefreshError):
                    error += _(' Try to <a href="%s">revoke and grant access</a> again') % reverse(
                        'jet-dashboard:update_module', kwargs={'pk': self.model.pk})
                self.error = mark_safe(error)


class GoogleAnalyticsVisitorsTotals(GoogleAnalyticsBase):
    """
    Google Analytics widget that shows total number of users, sessions and viewers for a particular period of time.
    Period may be following: Today, Last week, Last month, Last quarter, Last year
    """

    title = _('Google Analytics visitors totals')
    template = 'jet.dashboard/modules/google_analytics_visitors_totals.html'

    #: Which period should be displayed. Allowed values - integer of days
    period = None

    def __init__(self, title=None, period=None, **kwargs):
        kwargs.update({'period': period})
        super(GoogleAnalyticsVisitorsTotals, self).__init__(title, **kwargs)

    def init_with_context(self, context):
        result = self.api_ga()

        if result is not None:
            try:
                self.children.append({'title': _('users'), 'value': result['totalsForAllResults']['ga:users']})
                self.children.append({'title': _('sessions'), 'value': result['totalsForAllResults']['ga:sessions']})
                self.children.append({'title': _('views'), 'value': result['totalsForAllResults']['ga:pageviews']})
            except KeyError:
                self.error = _('Bad server response')


class GoogleAnalyticsVisitorsChart(GoogleAnalyticsBase):
    """
    Google Analytics widget that shows users/sessions/viewer chart for a particular period of time.
    Data is grouped by day, week or month
    Period may be following: Today, Last week, Last month, Last quarter, Last year
    """

    title = _('Google Analytics visitors chart')
    template = 'jet.dashboard/modules/google_analytics_visitors_chart.html'
    style = 'overflow-x: auto;'

    #: Which period should be displayed. Allowed values - integer of days
    period = None

    #: What data should be shown. Possible values: ``ga:users``, ``ga:sessions``, ``ga:pageviews``
    show = None

    #: Sets grouping of data. Possible values: ``day``, ``week``, ``month``
    group = None
    settings_form = GoogleAnalyticsChartSettingsForm

    class Media:
        js = ('jet.dashboard/vendor/chart.js/Chart.min.js', 'jet.dashboard/modules/google_analytics.js')

    def __init__(self, title=None, period=None, show=None, group=None, **kwargs):
        kwargs.update({'period': period, 'show': show, 'group': group})
        super(GoogleAnalyticsVisitorsChart, self).__init__(title, **kwargs)

    def settings_dict(self):
        settings = super(GoogleAnalyticsVisitorsChart, self).settings_dict()
        settings['show'] = self.show
        settings['group'] = self.group
        return settings

    def load_settings(self, settings):
        super(GoogleAnalyticsVisitorsChart, self).load_settings(settings)
        self.show = settings.get('show')
        self.group = settings.get('group')

    def init_with_context(self, context):
        result = self.api_ga(self.group)

        if result is not None:
            try:
                for data in result['rows']:
                    row_data = {}

                    i = 0
                    for column in result['columnHeaders']:
                        row_data[column['name'].replace(':', '_')] = data[i]
                        i += 1

                    date = self.get_grouped_date(row_data, self.group)
                    self.children.append((date, row_data[self.show.replace(':', '_')]))
            except KeyError:
                self.error = _('Bad server response')


class GoogleAnalyticsPeriodVisitors(GoogleAnalyticsBase):
    """
    Google Analytics widget that shows users, sessions and viewers for a particular period of time.
    Data is grouped by day, week or month
    Period may be following: Today, Last week, Last month, Last quarter, Last year
    """

    title = _('Google Analytics period visitors')
    template = 'jet.dashboard/modules/google_analytics_period_visitors.html'

    #: Which period should be displayed. Allowed values - integer of days
    period = None

    #: Sets grouping of data. Possible values: ``day``, ``week``, ``month``
    group = None
    contrast = False
    settings_form = GoogleAnalyticsPeriodVisitorsSettingsForm

    def __init__(self, title=None, period=None, group=None, **kwargs):
        kwargs.update({'period': period, 'group': group})
        super(GoogleAnalyticsPeriodVisitors, self).__init__(title, **kwargs)

    def settings_dict(self):
        settings = super(GoogleAnalyticsPeriodVisitors, self).settings_dict()
        settings['group'] = self.group
        return settings

    def load_settings(self, settings):
        super(GoogleAnalyticsPeriodVisitors, self).load_settings(settings)
        self.group = settings.get('group')

    def init_with_context(self, context):
        result = self.api_ga(self.group)

        if result is not None:
            try:
                for data in reversed(result['rows']):
                    row_data = {}

                    i = 0
                    for column in result['columnHeaders']:
                        row_data[column['name'].replace(':', '_')] = data[i]
                        i += 1

                    date = self.format_grouped_date(row_data, self.group)
                    self.children.append((date, row_data))
            except KeyError:
                self.error = _('Bad server response')



import json

from django.contrib.admin import ModelAdmin, RelatedFieldListFilter
from django.contrib.admin.utils import get_model_from_relation
from django.forms.utils import flatatt
from django.urls import reverse_lazy
from django.utils.encoding import smart_str
from django.utils.html import format_html

from jet.utils import format_widget_data


class RelatedFieldAjaxListFilter(RelatedFieldListFilter):
    template = 'jet/related_field_ajax_list_filter.html'
    ajax_attrs = None
    url = reverse_lazy('jet:model_lookup')
    widget_data = {}

    def has_output(self):
        return True

    def field_choices(self, field, request, model_admin: ModelAdmin):
        model = field.remote_field.model if hasattr(field, 'remote_field') else field.related_field.model
        app_label = model._meta.app_label
        model_name = model._meta.object_name

        self.ajax_attrs = format_html('{0}', flatatt({
            'data-app-label': app_label,
            'data-model': model_name,
            'data-ajax--url': self.url,
            'data-queryset--lookup': self.lookup_kwarg,
            'data-queryset--params': json.dumps(dict(request.GET.items())),
            'data-filter': f"By {self.title}",
            **format_widget_data(self.widget_data),
        }))

        if self.lookup_val is None:
            return []

        other_model = get_model_from_relation(field)
        if hasattr(field, 'rel'):
            rel_name = field.rel.get_related_field().name
        else:
            rel_name = other_model._meta.pk.name

        queryset = model._default_manager.filter(**{rel_name: self.lookup_val}).all()
        return [(x.pk, smart_str(x)) for x in queryset]


try:
    from collections import OrderedDict
    from django import forms
    from django.contrib.admin.widgets import AdminDateWidget
    from rangefilter.filter import DateRangeFilter as OriginalDateRangeFilter
    from django.utils.translation import ugettext as _


    class DateRangeFilter(OriginalDateRangeFilter):
        def get_template(self):
            return 'rangefilter/date_filter.html'

        def _get_form_fields(self):
            # this is here, because in parent DateRangeFilter AdminDateWidget
            # could be imported from django-suit
            return OrderedDict((
                (self.lookup_kwarg_gte, forms.DateField(
                    label='',
                    widget=AdminDateWidget(attrs={'placeholder': _('From date')}),
                    localize=True,
                    required=False
                )),
                (self.lookup_kwarg_lte, forms.DateField(
                    label='',
                    widget=AdminDateWidget(attrs={'placeholder': _('To date')}),
                    localize=True,
                    required=False
                )),
            ))

        @staticmethod
        def _get_media():
            css = [
                'style.css',
            ]
            return forms.Media(
                css={'all': ['range_filter/css/%s' % path for path in css]}
            )
except ImportError:
    pass



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['jet',
 'jet.dashboard',
 'jet.dashboard.migrations',
 'jet.dashboard.modules',
 'jet.dashboard.modules.google_analytics',
 'jet.dashboard.modules.yandex_metrika',
 'jet.dashboard.templatetags',
 'jet.management',
 'jet.management.commands',
 'jet.migrations',
 'jet.sidebar',
 'jet.templatetags']

package_data = \
{'': ['*'],
 'jet': ['locale/ar/LC_MESSAGES/django.po',
         'locale/ar/LC_MESSAGES/django.po',
         'locale/ar/LC_MESSAGES/djangojs.po',
         'locale/ar/LC_MESSAGES/djangojs.po',
         'locale/cs/LC_MESSAGES/django.po',
         'locale/cs/LC_MESSAGES/django.po',
         'locale/cs/LC_MESSAGES/djangojs.po',
         'locale/cs/LC_MESSAGES/djangojs.po',
         'locale/de/LC_MESSAGES/django.po',
         'locale/de/LC_MESSAGES/django.po',
         'locale/de/LC_MESSAGES/djangojs.po',
         'locale/de/LC_MESSAGES/djangojs.po',
         'locale/en/LC_MESSAGES/django.po',
         'locale/en/LC_MESSAGES/django.po',
         'locale/en/LC_MESSAGES/djangojs.po',
         'locale/en/LC_MESSAGES/djangojs.po',
         'locale/es/LC_MESSAGES/django.po',
         'locale/es/LC_MESSAGES/django.po',
         'locale/es/LC_MESSAGES/djangojs.po',
         'locale/es/LC_MESSAGES/djangojs.po',
         'locale/fa/LC_MESSAGES/django.po',
         'locale/fa/LC_MESSAGES/django.po',
         'locale/fa/LC_MESSAGES/djangojs.po',
         'locale/fa/LC_MESSAGES/djangojs.po',
         'locale/fr/LC_MESSAGES/django.po',
         'locale/fr/LC_MESSAGES/django.po',
         'locale/fr/LC_MESSAGES/djangojs.po',
         'locale/fr/LC_MESSAGES/djangojs.po',
         'locale/pl/LC_MESSAGES/django.po',
         'locale/pl/LC_MESSAGES/django.po',
         'locale/pl/LC_MESSAGES/djangojs.po',
         'locale/pl/LC_MESSAGES/djangojs.po',
         'locale/pt_BR/LC_MESSAGES/django.po',
         'locale/pt_BR/LC_MESSAGES/django.po',
         'locale/pt_BR/LC_MESSAGES/djangojs.po',
         'locale/pt_BR/LC_MESSAGES/djangojs.po',
         'locale/ru/LC_MESSAGES/django.po',
         'locale/ru/LC_MESSAGES/django.po',
         'locale/ru/LC_MESSAGES/djangojs.po',
         'locale/ru/LC_MESSAGES/djangojs.po',
         'locale/zh_CN/LC_MESSAGES/django.po',
         'static/admin/css/*',
         'static/admin/js/*',
         'static/admin/js/admin/*',
         'static/jet/css/icons/fonts/*',
         'static/jet/css/icons/style.css',
         'static/jet/css/jquery-ui/images/*',
         'static/jet/css/webfonts/*',
         'static/range_filter/css/*',
         'templates/admin/*',
         'templates/admin/edit_inline/*',
         'templates/admin/includes/*',
         'templates/jet/*',
         'templates/jet/sidebar/*',
         'templates/rangefilter/*'],
 'jet.dashboard': ['locale/ar/LC_MESSAGES/django.po',
                   'locale/ar/LC_MESSAGES/django.po',
                   'locale/ar/LC_MESSAGES/djangojs.po',
                   'locale/ar/LC_MESSAGES/djangojs.po',
                   'locale/cs/LC_MESSAGES/django.po',
                   'locale/cs/LC_MESSAGES/django.po',
                   'locale/cs/LC_MESSAGES/djangojs.po',
                   'locale/cs/LC_MESSAGES/djangojs.po',
                   'locale/de/LC_MESSAGES/django.po',
                   'locale/de/LC_MESSAGES/django.po',
                   'locale/de/LC_MESSAGES/djangojs.po',
                   'locale/de/LC_MESSAGES/djangojs.po',
                   'locale/en/LC_MESSAGES/django.po',
                   'locale/en/LC_MESSAGES/django.po',
                   'locale/en/LC_MESSAGES/djangojs.po',
                   'locale/en/LC_MESSAGES/djangojs.po',
                   'locale/es/LC_MESSAGES/django.po',
                   'locale/es/LC_MESSAGES/django.po',
                   'locale/es/LC_MESSAGES/djangojs.po',
                   'locale/es/LC_MESSAGES/djangojs.po',
                   'locale/fa/LC_MESSAGES/django.po',
                   'locale/fa/LC_MESSAGES/django.po',
                   'locale/fa/LC_MESSAGES/djangojs.po',
                   'locale/fa/LC_MESSAGES/djangojs.po',
                   'locale/fr/LC_MESSAGES/django.po',
                   'locale/fr/LC_MESSAGES/django.po',
                   'locale/fr/LC_MESSAGES/djangojs.po',
                   'locale/fr/LC_MESSAGES/djangojs.po',
                   'locale/pl/LC_MESSAGES/django.po',
                   'locale/pt-br/LC_MESSAGES/django.po',
                   'locale/pt-br/LC_MESSAGES/django.po',
                   'locale/pt-br/LC_MESSAGES/djangojs.po',
                   'locale/pt-br/LC_MESSAGES/djangojs.po',
                   'locale/ru/LC_MESSAGES/django.po',
                   'locale/ru/LC_MESSAGES/django.po',
                   'locale/ru/LC_MESSAGES/djangojs.po',
                   'locale/ru/LC_MESSAGES/djangojs.po',
                   'locale/zh_cn/LC_MESSAGES/django.po',
                   'static/jet.dashboard/modules/*',
                   'static/jet.dashboard/vendor/chart.js/*',
                   'templates/admin/*',
                   'templates/jet.dashboard/*',
                   'templates/jet.dashboard/modules/*']}

install_requires = \
['django-autocomplete-light>=3.5.1,<4.0.0',
 'django>=3,<4',
 'pydantic>=1.4,<2.0']

extras_require = \
{'google_analytics': ['google-api-python-client==1.4.1']}

setup_kwargs = {
    'name': 'djet2',
    'version': '1.0.9a6',
    'description': 'Modern template for Django admin interface with improved functionality',
    'long_description': "# Django JET\n\n![image](https://raw.githubusercontent.com/geex-arts/jet/static/logo.png)\n\n\n**what**|**where**\n-----|-----\nmaster tests|[![image][master-tests-badge]][master-tests]\ndev tests|[![image][dev-tests-badge]][dev-tests]\nlive demo|http://demo.jet.geex-arts.com/admin/\ndocumentation|http://jet.readthedocs.org/\nPyPi|https://pypi.org/project/djet2\n\n[master-tests]: https://github.com/vanyakosmos/djet2/actions?query=workflow%3Atest+branch%3Amaster\n[master-tests-badge]: https://github.com/vanyakosmos/djet2/workflows/test/badge.svg?branch=master\n[dev-tests]: https://github.com/vanyakosmos/djet2/actions?query=workflow%3Atest+branch%3Adev\n[dev-tests-badge]: https://github.com/vanyakosmos/djet2/workflows/test/badge.svg?branch=dev\n\n\n## Why Django JET?\n\n- New fresh look\n- Responsive mobile interface\n- Useful admin home page\n- Minimal template overriding\n- Easy integration\n- Themes support\n- Autocompletion\n- Handy controls\n\n## Screenshots\n\n![image](https://raw.githubusercontent.com/geex-arts/django-jet/static/screen1.png)\n\n![image](https://raw.githubusercontent.com/geex-arts/django-jet/static/screen2.png)\n\n![image](https://raw.githubusercontent.com/geex-arts/django-jet/static/screen3.png)\n\n## Installation\n\n- Download and install latest version of Django JET:\n\n```bash\npip install djet2\n```\n\n- Add 'jet' application to the INSTALLED\\_APPS setting of your Django project \nsettings.py file (note it should be before 'django.contrib.admin'):\n\n```python\nINSTALLED_APPS = (\n    ...,\n    'jet',\n    'django.contrib.admin',\n)\n```\n\n- Make sure `django.template.context_processors.request` context processor \nis enabled in settings.py (Django 1.8+ way):\n\n```python\nTEMPLATES = [\n    {\n        'BACKEND': 'django.template.backends.django.DjangoTemplates',\n        'DIRS': [],\n        'APP_DIRS': True,\n        'OPTIONS': {\n            'context_processors': [\n                ...,\n                'django.template.context_processors.request',\n                ...,\n            ],\n        },\n    },\n]\n```\n\n- Add URL-pattern to the urlpatterns of your Django project urls.py file \n(they are needed for related–lookups and autocompletes):\n\n```python\nurlpatterns = patterns(\n    '',\n    path('jet/', include('jet.urls', 'jet')),  # Django JET URLS\n    path('admin/', include(admin.site.urls)),\n    ...\n)\n```\n\n- Create database tables:\n\n```bash\npython manage.py migrate jet\n```\n\n- Collect static if you are in production environment:\n\n```bash\npython manage.py collectstatic\n```\n\n- Clear your browser cache:\n    - firefox/chrome: <kbd>Cmd</kbd>+<kbd>Shift</kbd>+<kbd>R</kbd> (<kbd>Ctrl</kbd>+<kbd>Shift</kbd>+<kbd>R</kbd>) for hard reload\n\n## Dashboard installation\n\nDashboard is located into a separate application. So after a typical JET \ninstallation it won't be active. To enable dashboard application follow these steps:\n\n- Add 'jet.dashboard' application to the INSTALLED\\_APPS setting of your Django \nproject settings.py file (note it should be before 'jet'):\n\n```python\nINSTALLED_APPS = (\n    ...,\n    'jet.dashboard',\n    'jet',\n    'django.contrib.admin',\n    ...,\n)\n```\n\n- Add URL-pattern to the urlpatterns of your Django project urls.py file \n(they are needed for related–lookups and autocompletes):\n\n```python\nurlpatterns = patterns(\n    '',\n    path('jet/', include('jet.urls', 'jet')),  # Django JET URLS\n    path('jet/dashboard/', include('jet.dashboard.urls', 'jet-dashboard')),  # Django JET dashboard URLS\n    path('admin/', include(admin.site.urls)),\n    ...,\n)\n```\n\n- **For Google Analytics widgets only** install python package:\n\n```bash\npip install djet2[google_analytics]\n# or \npip install google-api-python-client==1.4.1\n```\n\n- Create database tables:\n\n```bash\npython manage.py migrate dashboard\n```\n\n- Collect static if you are in production environment:\n\n```bash\npython manage.py collectstatic\n```\n",
    'author': 'Denis Kildishev',
    'author_email': 'support@jet.geex-arts.com',
    'maintainer': 'Bachynin Ivan',
    'maintainer_email': 'bachynin.i@gmail.com',
    'url': 'https://github.com/vanyakosmos/djet2',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from django.urls import re_path

from . import views


urlpatterns = [
    re_path(
        r'^yandex-metrika/grant/(?P<pk>\d+)/$',
        views.yandex_metrika_grant_view,
        name='yandex-metrika-grant'
    ),
    re_path(
        r'^yandex-metrika/revoke/(?P<pk>\d+)/$',
        views.yandex_metrika_revoke_view,
        name='yandex-metrika-revoke'
    ),
    re_path(
        r'^yandex-metrika/callback/$',
        views.yandex_metrika_callback_view,
        name='yandex-metrika-callback'
    ),
]



from django import forms
from django.utils.translation import gettext_lazy as _

from .base import DashboardModule


class LinkListItemForm(forms.Form):
    url = forms.CharField(label=_('URL'))
    title = forms.CharField(label=_('Title'))
    external = forms.BooleanField(label=_('External link'), required=False)


class LinkListSettingsForm(forms.Form):
    layout = forms.ChoiceField(label=_('Layout'), choices=(('stacked', _('Stacked')), ('inline', _('Inline'))))


class LinkList(DashboardModule):
    """
    List of links widget.

    Usage example:

    .. code-block:: python

        from django.utils.translation import gettext_lazy as _
        from jet.dashboard import modules
        from jet.dashboard.dashboard import Dashboard, AppIndexDashboard


        class CustomIndexDashboard(Dashboard):
            columns = 3

            def init_with_context(self, context):
                self.available_children.append(modules.LinkList)
                self.children.append(modules.LinkList(
                    _('Support'),
                    children=[
                        {
                            'title': _('Django documentation'),
                            'url': 'http://docs.djangoproject.com/',
                            'external': True,
                        },
                        {
                            'title': _('Django "django-users" mailing list'),
                            'url': 'http://groups.google.com/group/django-users',
                            'external': True,
                        },
                        {
                            'title': _('Django irc channel'),
                            'url': 'irc://irc.freenode.net/django',
                            'external': True,
                        },
                    ],
                    column=0,
                    order=0
                ))

    """

    title = _('Links')
    template = 'jet.dashboard/modules/link_list.html'

    #: Specify widget layout.
    #: Allowed values ``stacked`` and ``inline``.
    layout = 'stacked'

    #: Links are contained in ``children`` attribute which you can pass as constructor parameter
    #: to make your own preinstalled link lists.
    #:
    #: ``children`` is an array of dictinaries::
    #:
    #:     [
    #:          {
    #:              'title': _('Django documentation'),
    #:              'url': 'http://docs.djangoproject.com/',
    #:              'external': True,
    #:          },
    #:          ...
    #:     ]
    children = []
    settings_form = LinkListSettingsForm
    child_form = LinkListItemForm
    child_name = _('Link')
    child_name_plural = _('Links')

    def __init__(self, title=None, children=None, **kwargs):
        if children is None:
            children = []
        children = list(map(self.parse_link, children))
        kwargs.update({'children': children})
        super(LinkList, self).__init__(title, **kwargs)

    def settings_dict(self):
        return {
            'draggable': self.draggable,
            'deletable': self.deletable,
            'collapsible': self.collapsible,
            'layout': self.layout
        }

    def load_settings(self, settings):
        self.draggable = settings.get('draggable', self.draggable)
        self.deletable = settings.get('deletable', self.deletable)
        self.collapsible = settings.get('collapsible', self.collapsible)
        self.layout = settings.get('layout', self.layout)

    def store_children(self):
        return True

    def parse_link(self, link):
        if isinstance(link, (tuple, list)):
            link_dict = {'title': link[0], 'url': link[1]}
            if len(link) >= 3:
                link_dict['external'] = link[2]
            return link_dict
        elif isinstance(link, (dict,)):
            return link



import json

from django.template.loader import render_to_string

from jet.utils import LazyDateTimeEncoder, context_to_dict


class DashboardModule(object):
    """
    Base dashboard module class. All dashboard modules (widgets) should inherit it.
    """

    #: Path to widget's template. There is no need to extend such templates from any base templates.
    template = 'jet.dashboard/module.html'
    enabled = True

    #: Specify if module can be draggable or has static position.
    draggable = True

    #: Specify if module can be collapsed.
    collapsible = True

    #: Specify if module can be deleted.
    deletable = True
    show_title = True

    #: Default widget title that will be displayed for widget in the dashboard. User can change it later
    #: for every widget.
    title = ''

    #: Specify title url. ``None`` if title shouldn't be clickable.
    title_url = None
    css_classes = None

    #: HTML content that will be displayed before widget content.
    pre_content = None

    #: HTML content that will be displayed after widget content.
    post_content = None
    children = None

    #: A ``django.forms.Form`` class which may contain custom widget settings. Not required.
    settings_form = None

    #: A ``django.forms.Form`` class which may contain custom widget child settings, if it has any. Not required.
    child_form = None

    #: Child name that will be displayed when editing module contents. Required if ``child_form`` set.
    child_name = None

    #: Same as child name, but plural.
    child_name_plural = None
    settings = None
    column = None
    order = None

    #: A boolean field which specify if widget should be rendered on dashboard page load or fetched
    #: later via AJAX.
    ajax_load = False

    #: A boolean field which makes widget ui color contrast.
    contrast = False

    #: Optional style attributes which will be applied to widget content container.
    style = False

    class Media:
        css = ()
        js = ()

    def __init__(self, title=None, model=None, context=None, **kwargs):
        if title is not None:
            self.title = title
        self.model = model
        self.context = context or {}

        for key in kwargs:
            if hasattr(self.__class__, key):
                setattr(self, key, kwargs[key])

        self.children = self.children or []

        if self.model:
            self.load_from_model()

    def fullname(self):
        return self.__module__ + "." + self.__class__.__name__

    def load_settings(self, settings):
        """
        Should be implemented to restore saved in database settings. Required if you have custom settings.
        """
        pass

    def load_children(self, children):
        self.children = children

    def store_children(self):
        """
        Specify if children field should be saved to database.
        """
        return False

    def settings_dict(self):
        """
        Should be implemented to save settings to database. This method should return ``dict`` which will be serialized
        using ``json``. Required if you have custom settings.
        """
        pass

    def dump_settings(self, settings=None):
        settings = settings or self.settings_dict()
        if settings:
            return json.dumps(settings, cls=LazyDateTimeEncoder)
        else:
            return ''

    def dump_children(self):
        if self.store_children():
            return json.dumps(self.children, cls=LazyDateTimeEncoder)
        else:
            return ''

    def load_from_model(self):
        self.title = self.model.title

        if self.model.settings:
            try:
                self.settings = json.loads(self.model.settings)
                self.load_settings(self.settings)
            except ValueError:
                pass

        if self.store_children() and self.model.children:
            try:
                children = json.loads(self.model.children)
                self.load_children(children)
            except ValueError:
                pass

    def init_with_context(self, context):
        """
        Allows you to load data and initialize module's state.
        """
        pass

    def get_context_data(self):
        context = context_to_dict(self.context)
        context.update({
            'module': self
        })
        return context

    def render(self):
        self.init_with_context(self.context)
        return render_to_string(self.template, self.get_context_data())



from typing import Any, Dict, List

from django.template import loader
from django.template.context import BaseContext

from .config import get_menu_items


class Section:
    template_name = None

    def __init__(self, template_name=None):
        self.template_name = self.template_name or template_name
        self.context = None  # will be populated in render()

    def get_context_data(self, context) -> Dict[str, Any]:
        if isinstance(context, BaseContext):
            context = context.flatten()
        return {**context, 'self': self}

    def get_extra_context(self):
        return {}

    def init_with_context(self, context):
        """Hook which will be called before rendering."""

    def render(self, context):
        self.context = self.get_context_data(context)
        self.init_with_context(self.context)
        self.context.update(self.get_extra_context())
        return loader.render_to_string(self.template_name, self.context)

    def popups(self) -> List['Section']:
        return []

    def render_popups(self):
        for section in self.popups():
            yield section.render(self.context)


class NavSection(Section):
    template_name = 'jet/sidebar/section_nav.html'


class AppPopup(Section):
    template_name = 'jet/sidebar/popup_app.html'

    def __init__(self, app, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.app = app

    def get_extra_context(self):
        return {'app': self.app}


class AppsSection(Section):
    template_name = 'jet/sidebar/section_apps.html'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.apps = []

    def get_extra_context(self):
        return {'app_list': self.apps}

    def init_with_context(self, context):
        self.apps = [
            app for app in get_menu_items(context)
            if app.has_perms
        ]

    def popups(self):
        return [AppPopup(app) for app in self.apps]


class BookmarkSection(Section):
    template_name = 'jet/sidebar/section_bookmarks.html'


class Sidebar(Section):
    template_name = 'jet/sidebar/sidebar.html'

    def __init__(self, *args, **kwargs):
        super(Sidebar, self).__init__(*args, **kwargs)
        self.sections: List[Section] = []

    def init_with_context(self, context):
        self.sections = [
            NavSection(),
            AppsSection(),
        ]
        if context['request'].user.has_perm('jet.change_bookmark'):
            self.sections.append(BookmarkSection())

    def popups(self):
        for item in self.sections:
            yield from item.popups()

    def render_sections(self):
        for section in self.sections:
            yield section.render(self.context)



from django.conf import settings

# Theme
JET_DEFAULT_THEME = getattr(settings, 'JET_DEFAULT_THEME', 'default')
JET_THEMES = getattr(settings, 'JET_THEMES', [])

# Side menu
JET_SIDE_MENU_CLS = getattr(settings, 'JET_SIDE_MENU_CLS', 'jet.sidebar.Sidebar')
JET_SIDE_MENU_COMPACT = getattr(settings, 'JET_SIDE_MENU_COMPACT', False)
JET_SIDE_MENU_ITEMS = getattr(settings, 'JET_SIDE_MENU_ITEMS', None)

# Improved usability
JET_CHANGE_FORM_SIBLING_LINKS = getattr(settings, 'JET_CHANGE_FORM_SIBLING_LINKS', True)
